/*
 *		ioboard.cpp
 *
 *		Module to handle inputs and outputs to the
 *		comupter boards for the ME 161 Haptic Display
 *
 *		-Allison Okamura 11/12/96
 *    -Allison Okamura 11/17/98
 */

/* Includes */
#include <stdio.h>
#include <dos.h>
#include <conio.h>
#include <iostream.h>

/* Defines */
#define HI        1
#define LO		   0
#define AD_BASE   0x300
#define DA_BASE   0x310
#define EOC_MASK  0x80
#define CHAN_0    0
#define AD_SCALE  0.0025
#define AD_OFFSET 0
#define AMP_SCALE (-303.75)
#define AMP_OFFSET (-4.0)
#define DA_SCALE  (4096/10.0)
#define DA_OFFSET 2048
#define DA_MAX    3000
#define DA_MIN    1000

/* Functions */

/*--------------------- scan_channels ----------------------*
 *  Specifies the number of channels to scan                *
 *----------------------------------------------------------*/
void scan_channels(int start_channel, int end_channel)
{
	unsigned char scanlims;

	// set channel
	scanlims = ((end_channel & 0x0f) << 4) + (start_channel & 0x0f);
	outp(AD_BASE+2, scanlims);
}

/*----------------------- init_board -----------------------*
 *  Function to initialize the CIO-DAS1600 board            *
 *----------------------------------------------------------*/
void init_board(void)
{
	//unsigned char status;
	int gain_code = 1;  // in bipolar, 0=+/-10v, 1=+/-5v, 2=+/-2.5v, 3=+/-1.25
   						  // in unipolar, 0=0to10v, 1=0to5v, 2=0to2.5v, 3=0to1.25

	// set channels to scan
   scan_channels(0,0);

	// set gain
	//status = inportb(AD_BASE+8);
	//cout <<"status: "<< status<<" (1=single-ended,0=differential)"<<endl;
	outp(AD_BASE+11, gain_code);  // 0=+/-10v, 1=5, 2=2.5 3=1.25
}

/*----------------------- read_ad --------------------------*
 *  Reads A/D line and returns value                        *
 *----------------------------------------------------------*/
long int read_ad(void)
{
	long int counts = 0;
	unsigned char lowbyte, highbyte;

   // start a conversion, by writing to reg 0
	outp(AD_BASE,0);
	while ((inportb(AD_BASE+8)&EOC_MASK) != 0);

	// get data bytes and assemble into word
	lowbyte = inportb(AD_BASE+0);
	highbyte = inportb(AD_BASE+1);
	counts = (highbyte << 4) + (lowbyte >> 4) + AD_OFFSET;  //shift zero position

	// print out channel read
	//gotoxy(1,14);
	//printf("Channels Read: %d\n",(inportb(AD_BASE)&0x0f));
	//printf("Status: %d\n",(inportb(AD_BASE+8))&0x20>>5);

	return(counts);
}

/*---------------------- output_da -------------------------*
 *  Outputs value to D/A board                              *
 *----------------------------------------------------------*/
void output_da(int channel, float current)
{
	int output, lo_output, hi_output;
	int lo_address, hi_address;

	output =  (int)((current-AMP_OFFSET)/AMP_SCALE*DA_SCALE) + DA_OFFSET;

	// safety limits
	if (output > DA_MAX) output = 3000;
	if (output < DA_MIN) output = 1000;

	lo_output = (output & 0x00FF);
	hi_output = (output & 0x0F00) >> 8;

	lo_address = DA_BASE + channel*2;
	hi_address = DA_BASE + 1 + channel*2;

	outp(lo_address, lo_output);
	outp(hi_address, hi_output);
}

